@@ -10,16 +10,18 @@ use I18N::LangTags::List;
 use Term::ANSIColor qw(:constants uncolor);
 use File::Find::Rule;
 use Makefile::Parser;
+use XML::LibXSLT;
+use XML::LibXML;
 
 use Publican::Localise;
 
 use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK $SINGLETON $LOCALISE);
 
-$VERSION = '2.1';
+$VERSION = '2.3';
 @ISA     = qw(Exporter AutoLoader);
 
 @EXPORT
-    = qw(dir_list debug_msg get_all_langs logger help_config maketext old2new);
+    = qw(dir_list debug_msg get_all_langs logger help_config maketext old2new new_tree);
 
 my $DEFAULT_CONFIG_FILE = 'publican.cfg';
 my $DEBUG               = undef;
@@ -90,6 +92,11 @@ my %PARAMS = (
         default => 'docs-5E',
 
     },
+    bridgehead_in_toc => {
+        descr => maketext(
+            'Display bridge head elements in the TOCs?'),
+        default => 0,
+    },
     chunk_first => {
         descr => maketext(
             'For HTML, should the first section be on the same page as its parent?'
@@ -130,7 +137,8 @@ my %PARAMS = (
         default => 0,
     },
     confidential_text => {
-        descr   => maketext('The text used to indicate content is confidential.'),
+        descr =>
+            maketext('The text used to indicate content is confidential.'),
         default => maketext('CONFIDENTIAL'),
     },
     cvs_root => {
@@ -149,7 +157,7 @@ my %PARAMS = (
         descr => maketext(
             'Name of this package. Fetched from title tag in xml_lang/TYPE_Info.xml'
         ),
-        constraint => '[^a-zA-Z_\-0-9.]',
+        constraint => '[^0-9a-zA-Z_\-\.\+]',
     },
     doc_url => {
         descr => maketext(
@@ -177,8 +185,9 @@ my %PARAMS = (
             maketext('Eclipse plugin name. Defaults to "$product $docname"'),
     },
     'ec_provider' => {
-        descr =>
-            maketext('Eclipse plugin provider. Defaults to "Publican-[_1]"', $VERSION),
+        descr => maketext(
+            'Eclipse plugin provider. Defaults to "Publican-[_1]"', $VERSION
+        ),
     },
     edition => {
         descr => maketext(
@@ -205,12 +214,14 @@ my %PARAMS = (
 
     },
     max_image_width => {
-        descr   => maketext('The maximum pixel width an image can be before it will be scaled.'),
+        descr => maketext(
+            'The maximum pixel width an image can be before it will be scaled.'
+        ),
         default => '444',
 
     },
     os_ver => {
-        descr   => maketext('The OS for which to to build packages.'),
+        descr   => maketext('The OS for which to build packages.'),
         default => '.el5',
 
     },
@@ -218,7 +229,7 @@ my %PARAMS = (
         descr => maketext(
             'Product this package covers. Fetched from productname tag in xml_lang/TYPE_Info.xml'
         ),
-        constraint => '[^a-zA-Z_\-]',
+        constraint => '[^0-9a-zA-Z_\-\.\+]',
 
     },
     prod_url => {
@@ -238,7 +249,7 @@ my %PARAMS = (
     },
     scm => {
         descr => maketext(
-            'Type of repository Remote Set Books are stored in. Supported types: SVN.'
+            'Type of repository in which books that form part of a remote set are stored. Supported types: SVN.'
         ),
 
     },
@@ -293,33 +304,44 @@ my %PARAMS = (
     },
     web_home => {
         descr => maketext(
-            'This is a Publican Website home page, not a standard book.'),
+            'This is a home page for a Publican-generated website, not a standard book.'
+        ),
+        alert =>
+            'web_home is deprecated and will be removed from Publican in the future. Use "web_type: home" instead.',
+    },
+    web_type => {
+        descr => maketext(
+            'This is a special page for a Publican-generated website, not a standard book. Valid types are home, product, and version.'
+        ),
     },
     web_host => {
         descr => maketext(
-            'This is a Publican Website host name, used for searches and Sitemap.Be sure to include the full path to your document tree. e.g. if your documentas are in the docs directory: http://www.example.com/docs'
+            'This is a host name for a Publican-generated website, used for searches and the Sitemap. Be sure to include the full path to your document tree. E.g. if your documents are in the docs directory: http://www.example.com/docs'
         ),
+        alert => 'web_host is deprecated and will be removed from Publican in the future. Use "host" in the web site configuration file instead.',
     },
     web_obsoletes =>
-        { descr => maketext('Packages to obsolete in web RPM.'), },
+        { descr => maketext('Packages to obsolete in web RPM.'),
+    },
     web_search => {
         descr => maketext(
-            'Override the default search form for a Publican Website. By default this will use Gogle search and do a site search if web_host is set.'
+            'Override the default search form for a Publican website. By default this will use Google search and do a site search if web_host is set.'
         ),
+        alert => 'web_search is deprecated and will be removed from Publican in the future. Use "search" in the web site configuration file instead.',
     },
     web_name_label => {
         descr => maketext(
-            'Override the book name, bottom level, menu label on a Publican Website.'
+            'Override the book name, bottom level, menu label on a Publican website.'
         ),
     },
     web_product_label => {
         descr => maketext(
-            'Override the product, top level, menu label on a Publican Website.'
+            'Override the product, top level, menu label on a Publican website.'
         ),
     },
     web_version_label => {
         descr => maketext(
-            'Override the version, middle level, menu label on a Publican Website. To hide this meu item set this to: UNUSED'
+            'Override the version, middle level, menu label on a Publican website. To hide this menu item set this to: UNUSED'
         ),
     },
     xml_lang => {
@@ -408,6 +430,12 @@ sub _load_config {
         {
             $config->param( $def, $PARAMS{$def}->{default} );
         }
+
+       # Output alerts about a parameter
+       if( defined $PARAMS{$def}->{alert}
+            and defined( $config->param($def) )) {
+           _alert($PARAMS{$def}->{alert} . "\n");
+       }
     }
 
     $config->param( 'common_config',  $common_config )  if $common_config;
@@ -691,6 +719,13 @@ sub debug_msg {
     return;
 }
 
+sub _alert {
+    my ($msg) = @_;
+    logger( $msg, RED );
+
+    return;
+}
+
 =head2 param
 
 Return the current value of a configuration parameter
@@ -726,8 +761,9 @@ sub help_config {
                 . maketext( "Default: [_1]", $PARAMS{$param}->{default} )
                 . "\n" )
             if ( defined( $PARAMS{$param}->{default} ) );
-        logger(   "\t\t"
-                . maketext( "Constraint: [_1]", $PARAMS{$param}->{constraint} )
+        logger( "\t\t"
+                . maketext( "Constraint: [_1]",
+                $PARAMS{$param}->{constraint} )
                 . "\n" )
             if ( defined( $PARAMS{$param}->{constraint} ) );
 
@@ -890,6 +926,193 @@ sub old2new {
     return;
 }
 
+=head2 get_abstract
+
+Return the abstract for the supplied langauge with all white space truncted.
+
+=cut
+
+sub get_abstract {
+    my ( $self, $args ) = @_;
+
+    my $lang = delete( $args->{lang} )|| croak( maketext("lang is a mandatory argument") );
+
+    if ( %{$args} ) {
+        croak(
+            maketext(
+                "unknown arguments: [_1]", join( ", ", keys %{$args} )
+            )
+        );
+    }
+
+    my $tmp_dir    = $self->param('tmp_dir');
+    my $info_file = "$tmp_dir/$lang/xml/" . $self->param('type') . '_Info.xml';
+
+    croak(maketext("abstract can not be calculated before building.")) unless (-f $info_file);
+
+    my $xsl_file = $self->param('common_config') . "/xsl/abstract.xsl";
+
+    my $abstract = $self->run_xslt({xml_file => $info_file, xsl_file => $xsl_file});
+
+    # tidy up white space
+    $abstract =~ s/^[ \t]*//gm;
+    $abstract =~ s/^\n\n+//;
+    $abstract =~ s/\n\n+$//;
+    $abstract =~ s/\n\n\n/\n\n/g;
+    $abstract =~ s/[ \t][ \t]+/ /gm;
+
+    return($abstract);
+}
+
+
+=head2 get_subtitle
+
+Return the subtitle for the supplied langauge with white space truncted.
+
+=cut
+
+sub get_subtitle {
+    my ( $self, $args ) = @_;
+
+    my $lang = delete( $args->{lang} )|| croak( maketext("lang is a mandatory argument") );
+
+    if ( %{$args} ) {
+        croak(
+            maketext(
+                "unknown arguments: [_1]", join( ", ", keys %{$args} )
+            )
+        );
+    }
+
+    my $tmp_dir    = $self->param('tmp_dir');
+    my $info_file = "$tmp_dir/$lang/xml/" . $self->param('type') . '_Info.xml';
+
+    croak(maketext("subtitle can not be calculated before building.")) unless (-f $info_file);
+
+    my $xsl_file = $self->param('common_config') . "/xsl/subtitle.xsl";
+
+    my $subtitle = $self->run_xslt({xml_file => $info_file, xsl_file => $xsl_file});
+
+    # tidy up white space
+    $subtitle =~ s/^\s*//gm;
+    $subtitle =~ s/\s+/ /;
+
+    return($subtitle);
+}
+
+=head2 run_xslt
+
+Apply the supplied xslt file to teh supplied XML and return a string of the output.
+
+=cut
+
+sub run_xslt {
+    my ( $self, $args ) = @_;
+    my $xml_file =  delete( $args->{xml_file} )
+        || croak( maketext("xml_file is a mandatory argument") );
+    my $xsl_file =  delete( $args->{xsl_file} )
+        || croak( maketext("xsl_file is a mandatory argument") );
+
+    if ( %{$args} ) {
+        croak(
+            maketext(
+                "unknown arguments: [_1]", join( ", ", keys %{$args} )
+            )
+        );
+    }
+
+    my $parser = XML::LibXML->new();
+    my $xslt   = XML::LibXSLT->new();
+    $parser->expand_entities(1);
+    my $source;
+    eval { $source = $parser->parse_file($xml_file); };
+
+    if ($@) {
+        if ( ref($@) ) {
+
+            # handle a structured error (XML::LibXML::Error object)
+            croak(
+                maketext(
+                    "FATAL ERROR: [_1]:[_2] in [_3] on line [_4]: [_5]",
+                    $@->domain(),
+                    $@->code(),
+                    $@->file(),
+                    $@->line(),
+                    $@->message(),
+                )
+            );
+        }
+        else {
+            croak( maketext( "FATAL ERROR: [_1]", $@ ) );
+        }
+    }
+
+    my $style_doc = $parser->parse_file($xsl_file);
+
+    if ( $^O eq 'MSWin32' ) {
+        eval { require Win32::TieRegistry; };
+        croak(
+            maketext(
+                "Failed to load Win32::TieRegistry module. Error: [_1]", $@
+            )
+        ) if ($@);
+
+        my $defualt_href
+            = 'http://docbook.sourceforge.net/release/xsl/current';
+        my $key = new Win32::TieRegistry( "LMachine\\Software\\Publican",
+            { Delimiter => "\\" } );
+
+        my $new_href = 'file:///D:/Data/temp/Redhat/docbook-xsl-1.75.2';
+        if ( $key and $key->GetValue("xsl_path") ) {
+            $new_href = 'file:///' . $key->GetValue("xsl_path");
+            $new_href =~ s/ /%20/g;
+            $new_href =~ s/\\/\//g;
+        }
+
+        my @nodelist = $style_doc->getElementsByTagName('xsl:import');
+        foreach my $node (@nodelist) {
+            my $href = $node->getAttribute('href');
+            debug_msg("changing $defualt_href to $new_href\n");
+            $href =~ s|$defualt_href|$new_href|;
+            $node->setAttribute( 'href', $href );
+        }
+    }
+
+    my $stylesheet = $xslt->parse_stylesheet($style_doc);
+    my $results = $stylesheet->transform( $source );
+    my $value;
+    eval { $value = $stylesheet->output_string($results) };
+
+    return($value);
+}
+
+=head2 new_tree
+
+Create a new XML::TreeBuilder object with the required attributes for DocBook.
+
+TODO: Make XmlClean use this.
+
+=cut
+
+sub new_tree {
+
+    my $store_comments = ( shift() || 0 );
+
+    my $xml_doc = XML::TreeBuilder->new(
+        { 'NoExpand' => "1", 'ErrorContext' => "2" } );
+    my $empty_element_map = $xml_doc->_empty_element_map;
+    $empty_element_map->{'xref'}       = 1;
+    $empty_element_map->{'index'}      = 1;
+    $empty_element_map->{'imagedata'}  = 1;
+    $empty_element_map->{'area'}       = 1;
+    $empty_element_map->{'ulink'}      = 1;
+    $empty_element_map->{'xi:include'} = 1;
+
+    $xml_doc->store_comments(1) if ($store_comments);
+
+    return ($xml_doc);
+}
+
 1;    # Magic true value required at end of module
 __END__
 
