@@ -3,6 +3,7 @@ package Publican::Builder;
 use utf8;
 use strict;
 use warnings;
+use 5.008;
 use Carp;
 use Config::Simple;
 use Publican;
@@ -33,7 +34,7 @@ use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);
 
 $VERSION = '0.2';
 @ISA     = qw(Exporter AutoLoader);
-@EXPORT  = qw(new_tree dtd_string);
+@EXPORT  = qw(dtd_string);
 
 my $INVALID = 1;
 
@@ -74,10 +75,21 @@ sub new {
     my ( $this, $args ) = @_;
     my $class = ref($this) || $this;
 
+    my $novalid = delete( $args->{novalid} ) || 0;
+
+    if ( %{$args} ) {
+        croak(
+            maketext(
+                "unknown arguments: [_1]", join( ", ", keys %{$args} )
+            )
+        );
+    }
+
     my $self = bless {}, $class;
 
     $self->{publican}   = Publican->new();
     $self->{translator} = Publican::Translate->new();
+    $self->{validate}   = !$novalid;
 
     return $self;
 }
@@ -86,9 +98,7 @@ sub new {
 
 Transform the source in to another format.
 
-FORMATS eclipse epub html html-single html-desktop pdf txt
-
-Valid formats: eclipse epub html html-single html-desktop pdf txt
+Valid formats: eclipse epub html html-single html-desktop man pdf txt
 
 =cut
 
@@ -138,6 +148,7 @@ sub build {
 
         # hmmm can't validate brand XML as it's incomplete
         if (    ( $type ne 'brand' )
+            and $self->{validate}
             and ( $self->validate_xml( { lang => $lang } ) == $INVALID ) )
         {
             logger(
@@ -176,6 +187,18 @@ sub build {
                     if ( $self->{publican}->param('web_home') ) {
                         $path = "publish/home/$lang";
                     }
+                    elsif ( $self->{publican}->param('web_type') ) {
+                        my $web_type = $self->{publican}->param('web_type');
+                        if ( $web_type =~ m/^home$/i ) {
+                            $path = "publish/home/$lang";
+                        }
+                        elsif ( $web_type =~ m/^product$/i ) {
+                            $path = "publish/home/$lang/$product";
+                        }
+                        elsif ( $web_type =~ m/^version$/i ) {
+                            $path = "publish/home/$lang/$product/$version";
+                        }
+                    }
                     elsif ( $format eq 'html-desktop' ) {
                         $path = "publish/desktop/$lang";
                     }
@@ -646,6 +669,8 @@ sub transform {
     my $ec_name             = $self->{publican}->param('ec_name');
     my $ec_id               = $self->{publican}->param('ec_id');
     my $ec_provider         = $self->{publican}->param('ec_provider');
+    my $product             = $self->{publican}->param('product');
+    my $bridgehead_in_toc   = $self->{publican}->param('bridgehead_in_toc');
 
     my $TAR_NAME
         = $self->{publican}->param('product') . '-'
@@ -663,7 +688,7 @@ sub transform {
         $dir = pushd("$tmp_dir/$lang");
         mkdir 'txt';
         my $TXT_FILE;
-        open( $TXT_FILE, ">:utf8", "txt/$docname.txt" )
+        open( $TXT_FILE, ">", "txt/$docname.txt" )
             || croak( maketext("Can't open file for text output!") );
         my $tree
             = HTML::TreeBuilder->new->parse_file("html-single/index.html");
@@ -693,13 +718,37 @@ sub transform {
         'use.extensions'             => 1,
         'tablecolumns.extensions'    => 1,
         'publican.version'           => "'$Publican::VERSION'",
+        'bridgehead.in.toc'          => $bridgehead_in_toc,
     );
 
     mkdir "$tmp_dir/$lang/$format";
 
+    my $pop_prod = $self->{publican}->param('product');
+    my $pop_ver  = $self->{publican}->param('version');
+    my $pop_name = $self->{publican}->param('docname');
+
     my $toc_path = '../../../..';
     $toc_path = '.' if ( $self->{publican}->param('web_home') );
 
+    if ( $self->{publican}->param('web_type') ) {
+        my $web_type = $self->{publican}->param('web_type');
+        if ( $web_type =~ m/^home$/i ) {
+            $toc_path = '.';
+            $pop_prod = undef;
+            $pop_ver  = undef;
+            $pop_name = undef;
+        }
+        elsif ( $web_type =~ m/^product$/i ) {
+            $toc_path = '..';
+            $pop_ver  = undef;
+            $pop_name = undef;
+        }
+        elsif ( $web_type =~ m/^version$/i ) {
+            $toc_path = '../..';
+            $pop_name = undef;
+        }
+    }
+
     if ( $format eq 'html-single' ) {
 
         $dir = pushd("$tmp_dir/$lang/$format");
@@ -709,6 +758,9 @@ sub transform {
         $xslt_opts{'package'} = "'$TAR_NAME-$lang-$RPM_VERSION-$RPM_RELEASE'";
         $xslt_opts{'embedtoc'} = $embedtoc;
         $xslt_opts{'tocpath'}  = "'$toc_path'";
+        $xslt_opts{'pop_prod'} = "'$pop_prod'" if ($pop_prod);
+        $xslt_opts{'pop_ver'}  = "'$pop_ver'" if ($pop_ver);
+        $xslt_opts{'pop_name'} = "'$pop_name'" if ($pop_name);
     }
     elsif ( $format eq 'html-desktop' ) {
         $xsl_file = $common_config . "/xsl/html-single.xsl";
@@ -732,6 +784,9 @@ sub transform {
         $xslt_opts{'tocpath'}              = "'$toc_path'";
         $xslt_opts{'chunk.first.sections'} = $chunk_first;
         $xslt_opts{'chunk.section.depth'}  = $chunk_section_depth;
+        $xslt_opts{'pop_prod'}             = "'$pop_prod'" if ($pop_prod);
+        $xslt_opts{'pop_ver'}              = "'$pop_ver'" if ($pop_ver);
+        $xslt_opts{'pop_name'}             = "'$pop_name'" if ($pop_name);
     }
     elsif ( ( $format =~ /^pdf/ ) and ( -f $xsl_file ) ) {
         $dir = pushd("$tmp_dir/$lang/xml");
@@ -745,6 +800,9 @@ sub transform {
         $xslt_opts{'eclipse.plugin.provider'} = "'$ec_provider'";
         $dir = pushd("$tmp_dir/$lang/$format");
     }
+    elsif ( $format eq 'man' ) {
+        $dir = pushd("$tmp_dir/$lang/$format");
+    }
     else {
         croak( maketext( "Unknown format: [_1]", $format ) );
     }
@@ -761,6 +819,8 @@ sub transform {
     XML::LibXSLT->register_function( 'urn:perl', 'highlight', \&highlight );
     XML::LibXSLT->register_function( 'urn:perl', 'insertCallouts',
         \&insertCallouts );
+    XML::LibXSLT->register_function( 'urn:perl', 'numberLines',
+        \&numberLines );
     XML::LibXSLT->max_depth(1000);
 
     my $security = XML::LibXSLT::Security->new();
@@ -771,7 +831,30 @@ sub transform {
 
     $parser->expand_xinclude(1);
     $parser->expand_entities(1);
-    my $source    = $parser->parse_file("../xml/$docname.xml");
+##    my $source = $parser->parse_file("../xml/$docname.xml");
+    my $source;
+    eval { $source = $parser->parse_file("../xml/$docname.xml"); };
+
+    if ($@) {
+        if ( ref($@) ) {
+
+            # handle a structured error (XML::LibXML::Error object)
+            croak(
+                maketext(
+                    "FATAL ERROR: [_1]:[_2] in [_3] on line [_4]: [_5]",
+                    $@->domain(),
+                    $@->code(),
+                    $@->file(),
+                    $@->line(),
+                    $@->message(),
+                )
+            );
+        }
+        else {
+            croak( maketext( "FATAL ERROR: [_1]", $@ ) );
+        }
+    }
+
     my $style_doc = $parser->parse_file($xsl_file);
 
     if ( $^O eq 'MSWin32' ) {
@@ -894,6 +977,10 @@ sub transform {
         );
         $dir = undef;
     }
+    elsif ( $format eq 'man' ) {
+
+        # NO-OP?
+    }
     else {
         $dir = undef;
         dircopy( "$tmp_dir/$lang/xml/images",
@@ -1210,13 +1297,13 @@ sub insertCallouts {
                     my $col = $child->getAttribute('coords')
                         || carp(
                         maketext("'area' requires a 'coords' attribute.") );
-                    if( $col =~ m/^(\d+)\s+(\d+)$/ ) {
+                    if ( $col =~ m/^(\d+)\s+(\d+)$/ ) {
                         $col = $1;
                         $pos = $2;
                     }
 
                     push( @{ $callout{$col}{lines} }, $index );
-                    $callout{$col}{'pos'} = $pos ;
+                    $callout{$col}{'pos'} = $pos;
                 }
             }
         }
@@ -1226,7 +1313,7 @@ sub insertCallouts {
             my $col = $node->getAttribute('coords')
                 || carp( maketext("'area' requires a 'coords' attribute.") );
 
-            if( $col =~ m/^(\d+)\s+(\d+)$/ ) {
+            if ( $col =~ m/^(\d+)\s+(\d+)$/ ) {
                 $col = $1;
                 $pos = $2;
             }
@@ -1321,7 +1408,7 @@ sub insertCallouts {
             # if the position requested is less than the line length,
             # use the calculated position instead
             my $pos = $callout{$count}{'pos'};
-            $pos = $position if($pos < $callout{$count}{'length'});
+            $pos = $position if ( $pos < $callout{$count}{'length'} );
 
             my $padding = $pos - ( $callout{$count}{'length'} || 0 );
             $out_string .= " " x $padding;
@@ -1367,6 +1454,42 @@ sub insertCallouts {
     return ($verbatim);
 }
 
+=head2  numberLines
+
+perl_numberLines XSL function for numbering lines.
+
+Returns: Modified input tree, which is DocBook XML.
+
+=cut
+
+sub numberLines {
+    my $content = shift();
+
+    my $parser = XML::LibXML->new();
+    $parser->expand_entities(0);
+
+    my $text      = $content->string_value();
+    my $num_lines = () = ( $text =~ /^/mg );
+    my $format    = '%' . length("$num_lines") . 's:' . chr(160);
+
+    my $out_string = $text;
+    my $count      = 0;
+    $out_string =~ s/^/sprintf("$format",$count++)/egm;
+
+    # this gives an XML::LibXML::DocumentFragment
+    my $list = $parser->parse_balanced_chunk($out_string);
+
+    # remove the input node
+    $content->shift;
+
+    # append the marked-up nodes
+    foreach my $node ( $list->childNodes() ) {
+        $content->push($node);
+    }
+
+    return ($content);
+}
+
 =head2 package_brand
 
 Create the structure for the distributed files and save it as a tar.gz file
@@ -1451,9 +1574,13 @@ sub package_home {
     my $release    = $self->{publican}->param('release');
     my $xml_lang   = $self->{publican}->param('xml_lang');
     my $type       = $self->{publican}->param('type');
+    my $web_type   = $self->{publican}->param('web_type') || 'home';
 
     my $name_start = "$docname";
-    my $tardir     = "$name_start-web-home-$edition";
+    $name_start = "$product-$docname"          if ( $web_type eq 'product' );
+    $name_start = "$product-$docname-$version" if ( $web_type eq 'version' );
+
+    my $tardir = "$name_start-web-$web_type-$edition";
 
     mkpath("$tmp_dir/tar/$tardir");
     mkpath("$tmp_dir/rpm");
@@ -1486,8 +1613,6 @@ sub package_home {
     my $doc_url = $self->{publican}->param('doc_url');
     my $src_url = $self->{publican}->param('src_url');
     my $os_ver  = $self->{publican}->param('os_ver');
-    my $search  = $self->{publican}->param('web_search');
-    my $host    = $self->{publican}->param('web_host');
     my $log     = $self->change_log();
 
     my %xslt_opts = (
@@ -1503,8 +1628,7 @@ sub package_home {
         'src_url'    => $src_url,
         'log'        => $log,
         tmpdir       => $tmp_dir,
-        web_search   => $search,
-        web_host     => $host,
+        web_type     => $web_type,
     );
 
     logger(
@@ -1521,7 +1645,7 @@ sub package_home {
         XML::LibXSLT::xpath_to_string(%xslt_opts) );
 
     my $outfile;
-    my $spec_name = "$tmp_dir/rpm/$name_start-web-home.spec";
+    my $spec_name = "$tmp_dir/rpm/$name_start-web-$web_type.spec";
 
     open( $outfile, ">:utf8", "$spec_name" )
         || croak( maketext( "Can't open spec file: [_1]", $@ ) );
@@ -1529,7 +1653,7 @@ sub package_home {
     close($outfile);
 
     $self->build_rpm(
-        {   spec   => "$tmp_dir/rpm/$name_start-web-home.spec",
+        {   spec   => "$tmp_dir/rpm/$name_start-web-$web_type.spec",
             binary => $binary
         }
     );
@@ -1658,10 +1782,14 @@ sub package {
 
         if ( -f $po_file ) {
             my $PO;
+            my $real_value;
             open( $PO, "<:utf8", $po_file )
                 || croak( maketext( "Can't open PO file: [_1]", $po_file ) );
-            while (<$PO>) {
-                if (/"Project-Id-Version:\s*(\d*).*"$/) {
+            foreach my $line (<$PO>) {
+                if ( $line =~ /^"Project-Id-Version:/ ) {
+                    $line =~ /^"Project-Id-Version:\s*(.*)"$/;
+                    $real_value = $1;
+                    $line =~ /^"Project-Id-Version:\s*([\d.]*).*"$/;
                     $release = $1;
                     last;
                 }
@@ -1670,10 +1798,11 @@ sub package {
 
             croak(
                 maketext(
-                    "Project-Id-Version in [_1] is not a valid release value.",
+                    "Project-Id-Version '[_1]' in [_2] is not a valid release value.",
+                    $real_value,
                     $po_file
                 )
-            ) unless defined $release;
+            ) unless defined $release && $release ne "";
         }
         else {
             croak(
@@ -1747,18 +1876,71 @@ sub package {
     my $xsl_file      = $common_config . "/xsl/web-spec.xsl";
     $xsl_file = $common_config . "/xsl/dt_htmlsingle_spec.xsl" if ($desktop);
     $xsl_file =~ s/"//g;    # windows
-    my $license       = $self->{publican}->param('license');
-    my $brand         = lc( $self->{publican}->param('brand') );
-    my $doc_url       = $self->{publican}->param('doc_url');
-    my $src_url       = $self->{publican}->param('src_url');
-    my $dt_obsoletes  = $self->{publican}->param('dt_obsoletes') || "";
-    my $web_obsoletes = $self->{publican}->param('web_obsoletes') || "";
-    my $os_ver        = $self->{publican}->param('os_ver');
-    my $translation   = ( $lang ne $xml_lang );
-    my $language      = code2language( substr( $lang, 0, 2 ) );
+    my $license           = $self->{publican}->param('license');
+    my $brand             = lc( $self->{publican}->param('brand') );
+    my $doc_url           = $self->{publican}->param('doc_url');
+    my $src_url           = $self->{publican}->param('src_url');
+    my $dt_obsoletes      = $self->{publican}->param('dt_obsoletes') || "";
+    my $web_obsoletes     = $self->{publican}->param('web_obsoletes') || "";
+    my $os_ver            = $self->{publican}->param('os_ver');
+    my $translation       = ( $lang ne $xml_lang );
+    my $language          = code2language( substr( $lang, 0, 2 ) );
+    my $web_product_label = $self->{publican}->param('web_product_label')
+        || "";
+    my $web_version_label = $self->{publican}->param('web_version_label')
+        || "";
+    my $web_name_label = $self->{publican}->param('web_name_label') || "";
+
+    if ( $lang ne $xml_lang ) {
+        my $xml_file = "$tmp_dir/$lang/xml/$type" . '_Info.xml';
+        croak( maketext( "Can't locate required file: [_1]", $xml_file ) )
+            if ( !-f $xml_file );
+
+        my $xml_doc = XML::TreeBuilder->new();
+        $xml_doc->parse_file($xml_file);
+
+        # BUGBUG can't translate overridden labels :(
+        unless ($web_product_label) {
+            $web_product_label = eval {
+                $xml_doc->root()->look_down( "_tag", "productname" )
+                    ->as_text();
+            };
+            if ($@) {
+                croak maketext("productname not found in Info file");
+            }
+            $web_product_label =~ s/\s/_/g;
+        }
+
+        unless ($web_name_label) {
+            $web_name_label = eval {
+                $xml_doc->root()->look_down( "_tag", "title" )->as_text();
+            };
+            if ($@) {
+                croak maketext("title not found in Info file");
+            }
+            $web_name_label =~ s/\s/_/g;
+        }
+    }
+
+    $web_product_label =~ s/'/\\'/g;
+    $web_name_label =~ s/'/\\'/g;
+    $web_version_label =~ s/'/\\'/g;
 
     my $log = $self->change_log();
-    my $abstract = $self->abstract( { lang => $lang } );
+
+    my $full_abstract = $self->{publican}->get_abstract( { lang => $lang } );
+
+    # Wrap description for RPM style requirements
+    $columns = 68;
+    my $abstract = fill( "", "", $full_abstract );
+    $columns = $DEFAULT_WRAP;
+
+    # Escape single quotes to prevent bash breaking
+    $full_abstract =~ s/'/\\'/g;
+
+    my $full_subtitle = $self->{publican}->get_subtitle( { lang => $lang } );
+    $full_subtitle =~ s/'/\\'/g;
+    chomp($full_subtitle);
 
     my %xslt_opts = (
         'book-title'  => $name_start,
@@ -1780,6 +1962,11 @@ sub package {
         abstract      => $abstract,
         tmpdir        => $tmp_dir,
         ICONS         => ( -e "$xml_lang/icons" ? 1 : 0 ),
+        product_label => $web_product_label,
+        version_label => $web_version_label,
+        name_label    => $web_name_label,
+        full_abstract => $full_abstract,
+        full_subtitle => $full_subtitle,
     );
 
     logger(
@@ -1834,7 +2021,7 @@ sub change_log {
 
     $xml_doc->root()->look_down( "_tag", "revision" ) || croak(
         maketext(
-            "Missing mandatory field '[_1]' in revision histroy.", 'revision'
+            "Missing mandatory field '[_1]' in revision history.", 'revision'
         )
     );
 
@@ -1843,19 +2030,19 @@ sub change_log {
 
         my $node = $revision->look_down( '_tag', 'date' ) || croak(
             maketext(
-                "Missing mandatory field '[_1]' in revision histroy.", 'date'
+                "Missing mandatory field '[_1]' in revision history.", 'date'
             )
         );
         my $in_date = $node->as_trimmed_text();
         my $dt      = DateTime::Format::DateParse->parse_datetime($in_date)
             || croak(
-            maketext( "Invalid date: '[_1]' in revision histroy.", $in_date )
+            maketext( "Invalid date: '[_1]' in revision history.", $in_date )
             );
         my $date = $dt->strftime("%a %b %e %Y");
 
         $node = $revision->look_down( '_tag', 'firstname' ) || croak(
             maketext(
-                "Missing mandatory field '[_1]' in revision histroy.",
+                "Missing mandatory field '[_1]' in revision history.",
                 'firstname'
             )
         );
@@ -1863,7 +2050,7 @@ sub change_log {
 
         $node = $revision->look_down( '_tag', 'surname' ) || croak(
             maketext(
-                "Missing mandatory field '[_1]' in revision histroy.",
+                "Missing mandatory field '[_1]' in revision history.",
                 'surname'
             )
         );
@@ -1871,20 +2058,32 @@ sub change_log {
 
         $node = $revision->look_down( '_tag', 'email' ) || croak(
             maketext(
-                "Missing mandatory field '[_1]' in revision histroy.", 'email'
+                "Missing mandatory field '[_1]' in revision history.", 'email'
             )
         );
         my $email = $node->as_trimmed_text();
 
         $node = $revision->look_down( '_tag', 'revnumber' ) || croak(
             maketext(
-                "Missing mandatory field '[_1]' in revision histroy.",
+                "Missing mandatory field '[_1]' in revision history.",
                 'revnumber'
             )
         );
+
         my $revnumber = $node->as_trimmed_text();
 
-        $log .= sprintf( "* %s %s %s <%s> %s\n",
+        unless ( $revnumber =~ m/-\d/ ) {
+            logger(
+                maketext(
+                    "WARNING: revnumber '[_1]' does not match required format '(\\d.)*-(\\d.)*'. e.g. '[_1]-0'.\nWARNING: Appending '-0' to revnumber for change log use to avoid creating invalid change log entries.\nWARNING: This automated handling is deprecated and will be removed from Publican in a future version, you should correct your revnumber's before this or your future builds will fail.\n\n",
+                    $revnumber
+                ),
+                RED
+            );
+            $revnumber .= '-0';
+        }
+
+        $log .= sprintf( "* %s %s %s <%s> - %s\n",
             $date, $firstname, $surname, $email, $revnumber );
 
         $revision->look_down( '_tag', 'member' )
@@ -1903,38 +2102,6 @@ sub change_log {
     return ($log);
 }
 
-=head2 abstract
-
-Generate an RPM style description from $lang/$TYPE_Info.xml
-
-=cut
-
-sub abstract {
-    my ( $self, $args ) = @_;
-
-    my $lang = delete( $args->{lang} )
-        || croak("lang is a mandatory argument");
-
-    if ( %{$args} ) {
-        croak "unknown args: " . join( ", ", keys %{$args} );
-    }
-
-    my $tmp_dir = $self->{publican}->param('tmp_dir');
-    my $type    = $self->{publican}->param('type');
-
-    my $xml_doc = XML::TreeBuilder->new();
-    $xml_doc->parse_file( "$tmp_dir/$lang/xml/$type" . '_Info.xml' );
-
-    my $abstract = $xml_doc->root()->look_down( "_tag", "abstract" )
-        || croak( maketext( "Missing mandatory field '[_1]'.", 'abstract' ) );
-
-    $columns = 68;
-    my $text = fill( "", "", $abstract->as_text() );
-    $columns = $DEFAULT_WRAP;
-    $text =~ s/^\s*//s;
-    return ($text);
-}
-
 =head2 get_books
 
 Fetch all the books for a set from a repo.
@@ -2070,33 +2237,6 @@ sub build_set_books {
     return;
 }
 
-=head2 new_tree
-
-Create a new XML::TreeBuilder object with the required attributes for DocBook.
-
-TODO: Make XmlClean use this.
-
-=cut
-
-sub new_tree {
-
-    my $store_comments = ( shift() || 0 );
-
-    my $xml_doc = XML::TreeBuilder->new(
-        { 'NoExpand' => "1", 'ErrorContext' => "2" } );
-    my $empty_element_map = $xml_doc->_empty_element_map;
-    $empty_element_map->{'xref'}       = 1;
-    $empty_element_map->{'index'}      = 1;
-    $empty_element_map->{'imagedata'}  = 1;
-    $empty_element_map->{'area'}       = 1;
-    $empty_element_map->{'ulink'}      = 1;
-    $empty_element_map->{'xi:include'} = 1;
-
-    $xml_doc->store_comments(1) if ($store_comments);
-
-    return ($xml_doc);
-}
-
 =head2 dtd_string
 
 Returns a valid DTD for the DocBook tag supplied.
