@@ -3,6 +3,7 @@ package Publican::Translate;
 use utf8;
 use strict;
 use warnings;
+use 5.008;
 use Carp;
 use Publican;
 use Publican::Builder;
@@ -11,6 +12,7 @@ use File::Path;
 use Term::ANSIColor qw(:constants);
 use DateTime;
 use Locale::PO;
+use XML::TreeBuilder;
 
 use vars qw($VERSION);
 
@@ -18,10 +20,10 @@ $VERSION = '0.2';
 
 # What tags do we translate?
 my $TRANSTAGS
-    = qr/^(?:ackno|bridgehead|caption|conftitle|contrib|entry|firstname|glossterm|indexterm|jobtitle|keyword|label|lastname|lineannotation|lotentry|member|orgdiv|orgname|othername|para|productname|refclass|refdescriptor|refentrytitle|refmiscinfo|refname|refpurpose|releaseinfo|revremark|screeninfo|secondaryie|seealsoie|seeie|seg|segtitle|simpara|subtitle|surname|term|termdef|tertiaryie|title|titleabbrev|screen|programlisting|literallayout)$/;
+    = qr/^(?:ackno|bridgehead|caption|conftitle|contrib|entry|firstname|glossterm|indexterm|jobtitle|keyword|label|lastname|lineannotation|lotentry|member|orgdiv|orgname|othername|para|phrase|productname|refclass|refdescriptor|refentrytitle|refmiscinfo|refname|refpurpose|releaseinfo|revremark|screeninfo|secondaryie|seealsoie|seeie|seg|segtitle|simpara|subtitle|surname|term|termdef|tertiaryie|title|titleabbrev|screen|programlisting|literallayout)$/;
 
 # Blocks that contain translatable tags that need to be kept inline
-my $IGNOREBLOCKS = qr/^(?:footnote|citerefentry|indexterm)$/;
+my $IGNOREBLOCKS = qr/^(?:footnote|citerefentry|indexterm|productname|phrase)$/;
 
 # Preserve white space in these tags
 my $VERBATIM = qr/^(?:screen|programlisting|literallayout)$/;
@@ -155,10 +157,14 @@ sub po2xml {
 ##debug_msg("is utf8  key " . utf8::is_utf8($key) . "\n");
         my $msgref = $msgids->{$key};
 ##debug_msg("is utf8 msgref " . utf8::is_utf8($msgref->msgstr()) . "\n");
-        if ( $msgref->obsolete() || $msgref->fuzzy() ) {
+        if ( $msgref->obsolete() ) {
 ##            debug_msg("Deleting obsolete msg_id: $key\n");
             delete( $msgids->{$key} );
         }
+        if ( $msgref->fuzzy() ) {
+##            debug_msg("no fuzzy matches: $key\n");
+            $msgref->msgstr("");
+        }
     }
 
 ##debug_msg( "hash: " . join( "\n\n", keys( %{$msgids} ) ) . "\n\n" );
@@ -173,10 +179,13 @@ sub po2xml {
 ##    $text =~ s/&#9;//g;
     $text =~ s/&#38;([a-zA-Z-_0-9]+;)/&$1/g;
     $text =~ s/&#38;/&amp;/g;
-##    $text =~ s/&#60;/&lt;/g;
-##    $text =~ s/&#62;/&gt;/g;
-##    $text =~ s/&#34;/"/g;
-##    $text =~ s/&#39;/'/g;
+    $text =~ s/&#60;/&lt;/g;
+    $text =~ s/&#62;/&gt;/g;
+    $text =~ s/&#34;/"/g;
+    $text =~ s/&#39;/'/g;
+    $text =~ s/&quot;/"/g;
+    $text =~ s/&apos;/'/g;
+
     $out_doc->root()->delete();
 
     my $OUTDOC;
@@ -321,7 +330,7 @@ sub get_msgs {
         );
     }
 
-    my $trans_tree = XML::TreeBuilder->new();
+    my $trans_tree = XML::TreeBuilder->new( { 'NoExpand' => "1", 'ErrorContext' => "2" });
 
     my $trans_node;
 
@@ -336,7 +345,7 @@ sub get_msgs {
                 my $inner = $_[0];
 ## an index term NOT in a translatable tag should be translated as a block.
 ## An indexterm in a translatable tag should be translated inline
-                if ( $inner->tag() eq 'indexterm' ) {
+                if ( $inner->tag() =~ /indexterm|productname|phrase/ ) {
                     not defined(
                         $inner->look_up(
                             '_tag',
@@ -455,7 +464,32 @@ sub merge_msgs {
         $out_doc->look_down(
             '_tag',
             qr/$TRANSTAGS/,
-            sub { not defined( $_[0]->look_up( '_tag', qr/$IGNOREBLOCKS/ ) ) }
+##            sub { not defined( $_[0]->look_up( '_tag', qr/$IGNOREBLOCKS/ ) ) }
+            sub {
+                my $inner = $_[0];
+## an index term NOT in a translatable tag should be translated as a block.
+## An indexterm in a translatable tag should be translated inline
+                if ( $inner->tag() =~ /indexterm|productname|phrase/ ) {
+                    not defined(
+                        $inner->look_up(
+                            '_tag',
+                            qr/$IGNOREBLOCKS/,
+                            sub {
+                                $_[0]->tag() =~ /$TRANSTAGS/
+                                    && $inner->parent()
+                                    && $inner->parent()->tag()
+                                    =~ /$TRANSTAGS/;
+                            },
+                        )
+                    );
+                }
+                else {
+## Other IGNOREBLOCKS tags are completely ignored for translation structure.
+                    not defined(
+                        $inner->look_up( '_tag', qr/$IGNOREBLOCKS/ ) );
+
+                }
+            }
         )
         )
     {
@@ -568,9 +602,9 @@ sub translate {
     # If a tag has attributes we need to remove them for comparison as
     # the PO format does not allow this to be stored
 ## BUGBUG document this better
-    if ( $msgid =~ m/^<$tag([\t ]+[^>]+)>[\t ]*(.*)$/ ) {
+    if ( $msgid =~ m/^"<$tag([\t ]+[^>]+)>[\t ]*(.*)$/ ) {
         $attr_text = $1;
-        $msgid     = $2;
+        $msgid     = qq{"$2};
         $attr_text =~ s/\\//g;
     }
 
@@ -604,10 +638,27 @@ sub translate {
         }
         else {
 ##        debug_msg("WARNING: Un-translated message: '$msgid'\n");
+            if ( $msgids->{$msgid}->fuzzy() )
+            {    # BUGBUG TEST this is still set
+                logger( maketext("WARNING: Fuzzy message in PO file."), RED );
+            }
+            else {
+                logger(
+                    maketext("WARNING: Un-translated message in PO file."),
+                    RED );
+            }
+            logger( "\n" . $msgid . "\n\n", RED );
         }
     }
     else {
 ##         debug_msg("WARNING: Missing message: '\n$msgid\n'\n");
+        logger(
+            maketext(
+                "WARNING: Message missing from PO file, consider updating your POT and PO files."
+            ),
+            RED
+        );
+        logger( "\n" . $msgid . "\n\n", RED );
     }
     return;
 }
@@ -745,10 +796,12 @@ sub normalise {
     $norm =~ s/&amp;#x200B;/&#x200B;/g;
 ##    $norm =~ s/&#x200B; &#x200B;/ /g;
 ##    $norm =~ s/&#x200B; / /g;
-##    $norm =~ s/&#60;/&lt;/g;
-##    $norm =~ s/&#62;/&gt;/g;
-##    $norm =~ s/&#34;/"/g;
-##    $norm =~ s/&#39;/'/g;
+    $norm =~ s/&#60;/&lt;/g;
+    $norm =~ s/&#62;/&gt;/g;
+    $norm =~ s/&#34;/"/g;
+    $norm =~ s/&#39;/'/g;
+    $norm =~ s/&quot;/"/g;
+    $norm =~ s/&apos;/'/g;
 
 ##debug_msg("is utf8 norm: '" . utf8::is_utf8($norm) . "'\n");
 ##debug_msg("norm: $norm\n\n");
