@@ -3,6 +3,7 @@ package Publican::WebSite;
 use utf8;
 use warnings;
 use strict;
+use 5.008;
 
 use Carp qw(cluck croak);
 use Config::Simple;
@@ -12,13 +13,11 @@ use Template::Constants;
 use Locale::Language;
 use File::Find::Rule;
 use DateTime;
-## BUGBUG This requires splitting publican into 3 rpms to
-## allow translations to not conflict on systems
-## with web and builder installed
 use Publican;
 use Encode qw(is_utf8 decode_utf8 encode_utf8);
-
-#use Publican::Translate;
+use Time::localtime;
+use XML::Simple;
+use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
 
 our $VERSION = '1.3';
 
@@ -26,6 +25,7 @@ my $DB_NAME             = 'books';
 my $DEFAULT_LANG        = 'en-US';
 my $DEFAULT_TMPL_PATH   = '/usr/share/publican/templates';
 my $DEFAULT_CONFIG_FILE = '/etc/publican-website.cfg';
+my $DEFAULT_DUMP_FILE   = '/var/www/html/DUMP.xml';
 
 my %LANG_NAME = (
     'ar-SA'      => 'العربية',
@@ -42,6 +42,7 @@ my %LANG_NAME = (
     'el-GR'      => 'Ελληνικά',
     'en-US'      => 'English',
     'es-ES'      => 'Español',
+    'fa-IR'      => 'فارسی',
     'fi-FI'      => 'Suomi',
     'fr-FR'      => 'Français',
     'gu-IN'      => 'ગુજરાતી',
@@ -59,6 +60,7 @@ my %LANG_NAME = (
     'mr-IN'      => 'मराठी',
     'ms-MY'      => 'Melayu',
     'nb-NO'      => 'Norsk (bokmål)',
+    'nds-DE'     => 'Plattdüütsch',
     'nl-NL'      => 'Nederlands',
     'or-IN'      => 'ଓଡ଼ିଆ',
     'pa-IN'      => 'ਪੰਜਾਬੀ',
@@ -95,14 +97,20 @@ my %tmpl_strings = (
     'iframe'          => maketext(
         'This is an iframe, to view it upgrade your browser or enable iframe display.'
     ),
-    'Code'            => maketext('Code'),
-    'Products'        => maketext('Products'),
-    'Books'           => maketext('Books'),
-    'Versions'        => maketext('Versions'),
-    'Packages'        => maketext('Packages'),
-    'Total_Languages' => maketext('Total Languages'),
-    'Total_Packages'  => maketext('Total Packages'),
-    'Untranslated'    => maketext('Untranslated'),
+    'Code'             => maketext('Code'),
+    'Products'         => maketext('Products'),
+    'Books'            => maketext('Books'),
+    'Versions'         => maketext('Versions'),
+    'Packages'         => maketext('Packages'),
+    'Total_Languages'  => maketext('Total Languages'),
+    'Total_Packages'   => maketext('Total Packages'),
+    'Untranslated'     => maketext('Untranslated'),
+    'index_javascript' => maketext(
+        'This web site requires JavaScript and cookies to be enabled to function correctly.'
+    ),
+    'index_toc' => maketext('Click here to view a static Table of Contents'),
+    'ProductLinkTitle' => maketext('Information'),
+    ProductList        => maketext('Product List'),
 );
 
 sub new {
@@ -135,6 +143,12 @@ sub new {
             $site_config
         )
     );
+    my $host      = $config->param('host')      || 'http://localhost/';
+    my $search    = $config->param('search')    || undef;
+    my $title     = $config->param('title')     || 'Documentation';
+    my $dump      = $config->param('dump')      || undef;
+    my $dump_file = $config->param('dump_file') || $DEFAULT_DUMP_FILE;
+    my $zip_dump  = $config->param('zip_dump')  || undef;
 
     my $self = bless { db_file => $db_file }, $class;
 
@@ -146,6 +160,12 @@ sub new {
     $self->{toc_path}  = $toc_path;
     $self->{tmpl_path} = $tmpl_path;
     $self->{def_lang}  = $def_lang;
+    $self->{host}      = $host;
+    $self->{search}    = $search;
+    $self->{title}     = $title;
+    $self->{dump}      = $dump;
+    $self->{dump_file} = $dump_file;
+    $self->{zip_dump}  = $zip_dump;
 
     my $conf = { INCLUDE_PATH => $tmpl_path, };
 
@@ -193,11 +213,6 @@ sub _create_db {
         return;
     }
 
-    #TODO Table design
-    # ID, product, version,name, format
-    # ID unique key
-    # unique "product, version,name, format"?
-
     $self->_dbh->do(<<CREATE_TABLE);
 CREATE TABLE IF NOT EXISTS $DB_NAME (
 	ID INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
@@ -210,81 +225,15 @@ CREATE TABLE IF NOT EXISTS $DB_NAME (
 	version_label text(255),
 	name_label text(255),
 	update_date text(10),
+	subtitle text,
+	abstract text,
 	UNIQUE(language, product, version, name, format)
 )
 CREATE_TABLE
 
-    $self->_create_settings();
-
     return;
 }
 
-sub _create_settings {
-    my ( $self, $args ) = @_;
-
-    if ( $args && %{$args} ) {
-        croak( "unknown args: " . join( ", ", keys %{$args} ) );
-    }
-
-    $self->_dbh->do(<<CREATE_TABLE);
-CREATE TABLE IF NOT EXISTS settings (
-	host text(255),
-	search blob
-)
-CREATE_TABLE
-
-    return;
-}
-
-sub update_settings {
-    my ( $self, $args ) = @_;
-
-    my $host   = delete $args->{host};
-    my $search = delete $args->{search};
-
-    if ( $args && %{$args} ) {
-        croak( "unknown args: " . join( ", ", keys %{$args} ) );
-    }
-
-    $self->_create_settings();
-
-    my $sql = 'delete from settings ';
-    $self->_dbh()->do($sql);
-
-    $sql = 'INSERT INTO settings ';
-
-    my @cols;
-    my @vals;
-
-    if ($host) {
-        push( @cols, 'host' );
-        push( @vals, "'$host'" );
-    }
-
-    if ($search) {
-        push( @cols, 'search' );
-        push( @vals, "'$search'" );
-    }
-    $sql .= '(' . join( ',',        @cols ) . ') ';
-    $sql .= 'values (' . join( ',', @vals ) . ') ';
-
-    if ( $search || $host ) {
-        $self->_dbh()->do($sql);
-    }
-
-    return;
-}
-
-sub get_settings {
-    my ( $self, $args ) = @_;
-
-    $self->_create_settings();
-
-    my $hashref = $self->_dbh()->selectrow_hashref('SELECT * FROM settings');
-
-    return ($hashref);
-}
-
 sub _load_db {
     my $self = shift;
 
@@ -299,6 +248,70 @@ sub _load_db {
     return;
 }
 
+sub update_or_add_entry {
+    my ( $self, $arg ) = @_;
+
+    my $language = delete $arg->{language} || croak "language required";
+    my $product  = delete $arg->{product}  || croak "product required";
+    my $version  = delete $arg->{version}  || croak "version required";
+    my $name     = delete $arg->{name}     || croak "name required";
+    my $format   = delete $arg->{format}   || croak "format required";
+    my $product_label = delete $arg->{product_label};
+    my $version_label = delete $arg->{version_label};
+    my $name_label    = delete $arg->{name_label};
+    my $subtitle      = delete $arg->{subtitle};
+    my $abstract      = delete $arg->{abstract};
+
+    if ( %{$arg} ) {
+        croak "unknown args: " . join( ", ", keys %{$arg} );
+    }
+
+    my $rc;
+
+    my $id = $self->get_entry_id(
+        {   language => $language,
+            product  => $product,
+            version  => $version,
+            name     => $name,
+            format   => $format
+        }
+    );
+
+    if ($id) {
+        $rc = $self->update_entry(
+            {   ID            => $id,
+                language      => $language,
+                product       => $product,
+                version       => $version,
+                name          => $name,
+                format        => $format,
+                product_label => $product_label,
+                version_label => $version_label,
+                name_label    => $name_label,
+                subtitle      => $subtitle,
+                abstract      => $abstract,
+            }
+        );
+    }
+    else {
+        $rc = $self->add_entry(
+            {   language      => $language,
+                product       => $product,
+                version       => $version,
+                name          => $name,
+                format        => $format,
+                product_label => $product_label,
+                version_label => $version_label,
+                name_label    => $name_label,
+                subtitle      => $subtitle,
+                abstract      => $abstract,
+            }
+        );
+    }
+
+    return ($rc);
+}
+
 sub add_entry {
     my ( $self, $arg ) = @_;
 
@@ -310,6 +323,8 @@ sub add_entry {
     my $product_label = delete $arg->{product_label};
     my $version_label = delete $arg->{version_label};
     my $name_label    = delete $arg->{name_label};
+    my $subtitle      = delete $arg->{subtitle};
+    my $abstract      = delete $arg->{abstract};
 
     if ( %{$arg} ) {
         croak "unknown args: " . join( ", ", keys %{$arg} );
@@ -318,17 +333,19 @@ sub add_entry {
     my $update_date = DateTime->today()->ymd();
 
     $format = lc $format;
+    $abstract =~ s/\n/ /gm;
 
     my $sql = <<INSERT_ENTRY;
         INSERT INTO $DB_NAME 
-               (language, product, version, name, format, product_label, version_label, name_label, update_date) 
-               VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
+               (language, product, version, name, format, product_label, version_label, name_label, update_date, subtitle, abstract) 
+               VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
 INSERT_ENTRY
 
     return $self->_dbh->do(
-        $sql,           undef,       $language, $product,
-        $version,       $name,       $format,   $product_label,
-        $version_label, $name_label, $update_date
+        $sql,           undef,       $language,    $product,
+        $version,       $name,       $format,      $product_label,
+        $version_label, $name_label, $update_date, $subtitle,
+        $abstract
     );
 }
 
@@ -344,6 +361,8 @@ sub update_entry {
     my $product_label = delete $arg->{product_label};
     my $version_label = delete $arg->{version_label};
     my $name_label    = delete $arg->{name_label};
+    my $subtitle      = delete $arg->{subtitle};
+    my $abstract      = delete $arg->{abstract};
 
     if ( %{$arg} ) {
         croak "unknown args: " . join( ", ", keys %{$arg} );
@@ -352,17 +371,19 @@ sub update_entry {
     my $update_date = DateTime->today()->ymd();
 
     $format = lc $format;
+    $abstract =~ s/\n/ /gm;
 
     my $sql = <<INSERT_ENTRY;
         UPDATE $DB_NAME SET
-               language = ?, product = ?, version = ?, name = ?, format = ?, product_label = ?, version_label = ?, name_label = ?, update_date = ? 
+               language = ?, product = ?, version = ?, name = ?, format = ?, product_label = ?, version_label = ?, name_label = ?, update_date = ?, subtitle = ?, abstract = ?
                WHERE ID = $ID
 INSERT_ENTRY
 
     return $self->_dbh->do(
-        $sql,           undef,       $language, $product,
-        $version,       $name,       $format,   $product_label,
-        $version_label, $name_label, $update_date
+        $sql,           undef,       $language,    $product,
+        $version,       $name,       $format,      $product_label,
+        $version_label, $name_label, $update_date, $subtitle,
+        $abstract
     );
 }
 
@@ -417,6 +438,7 @@ sub get_entry_id {
 GET_ENTRY
 
     my $sth = $self->_dbh->prepare($sql);
+
     $sth->execute( $language, $product, $version, $name, $format );
     my $result = $sth->fetchrow();
 
@@ -448,7 +470,9 @@ sub get_hash_ref {
                product_label, 
                version_label, 
                name_label,
-               update_date
+               update_date,
+               subtitle,
+               abstract
           FROM $DB_NAME 
          WHERE (language = ? or language = ?)
       GROUP BY language, product, version, name, format
@@ -465,9 +489,9 @@ GET_LIST
     my %list;
 
     while (
-        my ($id,         $lang,   $product,       $version,
-            $name,       $format, $product_label, $version_label,
-            $name_label, $update_date
+        my ($id,         $lang,        $product,       $version,
+            $name,       $format,      $product_label, $version_label,
+            $name_label, $update_date, $subtitle,      $abstract
         )
         = $sth->fetchrow()
         )
@@ -487,6 +511,8 @@ GET_LIST
             if $product_label;
         $list{$product}{$version}{$name}{update_date} = $update_date
             || '2000-01-01';
+        $list{$product}{$version}{$name}{subtitle} = $subtitle;
+        $list{$product}{$version}{$name}{abstract} = $abstract;
     }
 
     $sth->finish();
@@ -507,8 +533,9 @@ sub get_lang_list {
     $langs = $self->_dbh->selectall_arrayref($sql);
 
     unless ( $langs->[0] ) {
-        print("No langs found, using default langs\n");
-        my @langs = ('en-US');
+
+        # No languages found, using default language: [_1]\n
+        my @langs = ( $self->{def_lang} );
         $langs->[0] = \@langs;
     }
 
@@ -525,9 +552,10 @@ sub regen_all_toc {
 
     my $langs = $self->get_lang_list();
 
-    my @fulltoc   = ();
-    my @toc_names = ();
-    my @urls      = ();
+    my @fulltoc        = ();
+    my @toc_names      = ();
+    my @urls           = ();
+    my @opds_lang_list = ();
 
     foreach my $lang ( @{$langs} ) {
         my %toc;
@@ -565,6 +593,27 @@ sub regen_all_toc {
         push( @fulltoc, \%toc );
         my %toc_name = ( 'toc_name' => $lang_name );
         push( @toc_names, \%toc_name );
+
+        my %opds_lang = (
+            url         => qq|$lang->[0]| . '/opds.xml',
+            id          => $self->{host} . '/' . qq|$lang->[0]| . '/opds.xml',
+            lang        => $lang->[0],
+            title       => $lang_name,
+            content     => "",
+            img         => "",
+            update_date => DateTime->now(),
+        );
+
+        $opds_lang{img}
+            = $self->{host} . '/'
+            . qq|$lang->[0]|
+            . '/images/cover_thumbnail.png'
+            if ( -f $self->{toc_path} . '/'
+            . qq|$lang->[0]|
+            . '/images/cover_thumbnail.png' );
+
+        push( @opds_lang_list, \%opds_lang );
+
     }
 
     my $vars = {
@@ -589,14 +638,48 @@ sub regen_all_toc {
         binmode => ':utf8'
     ) or croak( $self->{Template}->error() );
 
+    # regenerate main index.html
+    $vars = ();
+    my $locales = join( ",", map( qq|"$_->[0]"|, @{$langs} ) );
+    $vars = {
+        locales          => $locales,
+        title            => $self->{title},
+        def_lang         => $self->{def_lang},
+        index_javascript => $tmpl_strings{'index_javascript'},
+        index_toc        => $tmpl_strings{'index_toc'},
+    };
+    $self->{Template}->process(
+        'index.tmpl', $vars,
+        $self->{toc_path} . "/index.html",
+        binmode => ':utf8'
+    ) or croak( $self->{Template}->error() );
+
+    # This file contains all the langauges.
+    my $opds_vars;
+    $opds_vars->{langs}       = \@opds_lang_list;
+    $opds_vars->{update_date} = DateTime->now();
+    $opds_vars->{self}        = $self->{host} . "/opds.xml";
+    $opds_vars->{id}          = $self->{host} . "/opds.xml";
+    $opds_vars->{title}       = $self->{title};
+##    $opds_vars->{} = ;
+
+    $self->{Template}->process(
+        'opds-langs.tmpl', $opds_vars,
+        $self->{toc_path} . "/opds.xml",
+        binmode => ':utf8'
+    ) or croak( $self->{Template}->error() );
+
+    $self->xml_dump() if ( $self->{dump} );
     return;
 }
 
 sub version_sort {
+
     # X or X.Y
     if ( $a =~ /^(?:\d+|\d+\.\d+)$/ && $b =~ /^(?:\d+|\d+\.\d+)$/ ) {
         return $a <=> $b;
     }
+
     # X or X.Y Vs X.Y.Z
     elsif ( $a =~ /^(?:\d+|\d+\.\d+)$/ && $b =~ /^(\d+\.\d+)(.+)$/ ) {
         if ( $a gt $1 ) {
@@ -606,6 +689,7 @@ sub version_sort {
             return -1;
         }
     }
+
     # X.Y.Z Vs X or X.Y
     elsif ( $b =~ /^(?:\d+|\d+\.\d+)$/ && $a =~ /^(\d+\.\d+)(.+)$/ ) {
         if ( $1 ge $b ) {
@@ -615,6 +699,7 @@ sub version_sort {
             return -1;
         }
     }
+
     # X.Y.Z Vs X.Y.Z
     else {
         return $a cmp $b;
@@ -634,12 +719,8 @@ sub _regen_toc {
         croak "unknown args: " . join( ", ", keys %{$arg} );
     }
 
-    debug_msg("Processing lang: $language\n");
-
     my $vars = {};
 
-    my $settings = $self->get_settings();
-
     my $default_search = <<SEARCH;
 	<form target="_top" method="get" action="http://www.google.com/search">
 		<div class="search">
@@ -647,7 +728,7 @@ sub _regen_toc {
 			<input class="searchsub" type="submit" value="###Search###" />
 SEARCH
 
-    my $host = $settings->{host};
+    my $host = $self->{host};
 
     if ($host) {
         $default_search .= <<SEARCH;
@@ -682,7 +763,6 @@ SEARCH
         $row_data{'selected'}  = $selected;
         $row_data{'lang'}      = $lang->[0];
         $row_data{'lang_name'} = $lang_name;
-        debug_msg("lang name: $lang_name\n");
         push( @tmpl_langs, \%row_data );
     }
 
@@ -700,7 +780,7 @@ SEARCH
     $locale->encoding("UTF-8");
     $locale->textdomain("publican");
 
-    my $search = ( $settings->{search} || $default_search );
+    my $search = ( $self->{search} || $default_search );
     my $string = $locale->maketext("Search");
     $search =~ s/###Search###/$string/g;
     $vars->{'search'} = $search;
@@ -712,12 +792,31 @@ SEARCH
     }
 
     $vars->{untrans_lang} = $self->{def_lang};
+    my @opds_prod_list;
 
     foreach my $product ( sort( keys( %{$list2} ) ) ) {
+        my @opds_list;
         my $product_label = $product;
         my %prod_data;
         my @versions = ();
 
+        if ( -f "$self->{toc_path}/$language/$product/index.html" ) {
+            $prod_data{'product_icon'} = 1;
+            my $url = {
+                url         => qq|$host/$language/$product/index.html|,
+                update_date => ctime(
+                    (   stat(
+                            "$self->{toc_path}/$language/$product/index.html")
+                    )[9]
+                ),
+            };
+
+            push( @{$urls}, $url );
+        }
+        else {
+            $prod_data{'product_icon'} = 0;
+        }
+
         foreach my $version (
             reverse( sort( version_sort keys( %{ $list2->{$product} } ) ) ) )
         {
@@ -725,6 +824,28 @@ SEARCH
             my %ver_data;
             my @books         = ();
             my @untrans_books = ();
+            my $category      = $version_label;
+
+## BUGBUG TODO labels
+## It makes more sense to do an SQL query here for the labels instead of
+## doing it in the book loop.
+
+            if ( -f "$self->{toc_path}/$language/$product/$version/index.html"
+                )
+            {
+                $ver_data{'ver_icon'} = 1;
+                my $url = {
+                    url => qq|$host/$language/$product/$version/index.html|,
+                    update_date => ctime(
+                        (   stat(
+                                "$self->{toc_path}/$language/$product/$version/index.html"
+                            )
+                        )[9]
+                    ),
+                };
+
+                push( @{$urls}, $url );
+            }
 
             foreach
                 my $book ( sort( keys( %{ $list2->{$product}{$version} } ) ) )
@@ -793,6 +914,38 @@ SEARCH
                             "$self->{toc_path}/$lang/$product/$version/$type/$book"
                             );
                         $type_data{'ext'} = pop(@filelist);
+                        if ( $type_data{'ext'} ) {
+                            my %opds_url = (
+                                title => "$book_label",
+                                id =>
+                                    "$host/$lang/$product/$version/$type/$book/"
+                                    . $type_data{'ext'},
+                                lang => $language,
+                                update_date =>
+                                    $list2->{$product}{$version}{$book}
+                                    {update_date},
+                                url =>
+                                    "$host/$lang/$product/$version/$type/$book/"
+                                    . $type_data{'ext'},
+                                category => $category,
+                                summary  => (
+                                    $list2->{$product}{$version}{$book}
+                                        {subtitle} || ""
+                                ),
+                                content => (
+                                    $list2->{$product}{$version}{$book}
+                                        {abstract} || ""
+                                ),
+                            );
+                            $category = "";
+                            $opds_url{title} =~ s/_/ /g;
+                            my $img
+                                = "/$lang/$product/$version/html/$book/images/cover_thumbnail.png";
+                            $opds_url{img} = $self->{host} . $img
+                                if ( -f $self->{toc_path} . $img );
+
+                            push( @opds_list, \%opds_url );
+                        }
 ## hmm epub link for safari ...
 ##                        $type_data{prep} = "epub://$host/docs/$lang/";
                         $type_data{onclick} = 0;
@@ -839,11 +992,41 @@ SEARCH
             push( @versions, \%ver_data );
         }
 
-        $prod_data{'product'}       = $product;
-        $prod_data{'product_clean'} = $product_label;
+        $prod_data{'prod_link_title'} = $tmpl_strings{'ProductLinkTitle'};
+        $prod_data{'product'}         = $product;
+        $prod_data{'product_clean'}   = $product_label;
         $prod_data{'product_clean'} =~ s/_/ /g;
         $prod_data{'versions'} = \@versions;
         push( @products, \%prod_data );
+
+        # This file contains books for all versions of a product.
+        my $opds_vars;
+        $opds_vars->{urls}        = \@opds_list;
+        $opds_vars->{update_date} = DateTime->now();
+        $opds_vars->{self}        = "$host/$language/opds-$product.xml";
+        $opds_vars->{id}          = "$host/$language/opds-$product.xml";
+        $opds_vars->{title}       = $prod_data{'product_clean'};
+##        $opds_vars->{} = ;
+
+        $self->{Template}->process(
+            'opds.tmpl', $opds_vars,
+            $self->{toc_path} . "/$language/opds-$product.xml",
+            binmode => ':utf8'
+        ) or croak( $self->{Template}->error() );
+
+        my %opds_p_url = (
+            title       => $prod_data{'product_clean'},
+            id          => "$host/$language/$product/opds-$product.xml",
+            lang        => $language,
+            update_date => DateTime->now(),
+            url         => "opds-$product.xml",
+            content     => "",
+        );
+        my $img = "/$language/$product/images/cover_thumbnail.png";
+        $opds_p_url{img} = $self->{host} . $img
+            if ( -f $self->{toc_path} . $img );
+
+        push( @opds_prod_list, \%opds_p_url );
     }
 
     $vars->{'products'} = \@products;
@@ -854,6 +1037,21 @@ SEARCH
         binmode => ':utf8'
     ) or croak( $self->{Template}->error() );
 
+    # This file contains all products for this langauge.
+    my $opds_vars;
+    $opds_vars->{products}    = \@opds_prod_list;
+    $opds_vars->{update_date} = DateTime->now();
+    $opds_vars->{self}        = "$host/$language/opds.xml";
+    $opds_vars->{id}          = "$host/$language/opds.xml";
+    $opds_vars->{title}       = $tmpl_strings{ProductList};
+##    $opds_vars->{} = ;
+
+    $self->{Template}->process(
+        'opds-prods.tmpl', $opds_vars,
+        $self->{toc_path} . "/$language/opds.xml",
+        binmode => ':utf8'
+    ) or croak( $self->{Template}->error() );
+
     return \@products;
 }
 
@@ -924,23 +1122,6 @@ GET_COUNTS
         push( @lang_stats, \%stats );
     }
 
-    # This catches all the languages in the list without books installed.
-    foreach my $lang ( sort( keys(%LANG_NAME) ) ) {
-        if ( !defined $found_langs{$lang} ) {
-            $total_langs++;
-            my %stats;
-            $lang =~ m/^([^-_]*)/;
-            my $lang_name = code2language($1) || $lang;
-            $stats{'language'} = $lang_name;
-            $stats{'code'}     = $lang;
-            $stats{'packages'} = 0;
-            $stats{'books'}    = 0;
-            $stats{'products'} = 0;
-            $stats{'versions'} = 0;
-            push( @lang_stats, \%stats );
-        }
-    }
-
     foreach my $lang ( keys(%found_langs) ) {
 
         my $vars = {
@@ -1029,6 +1210,46 @@ GET_COUNT
     return;
 }
 
+sub xml_dump {
+    my ( $self, $arg ) = @_;
+
+    if ( $arg && %{$arg} ) {
+        croak "unknown args: " . join( ", ", keys %{$arg} );
+    }
+
+    my $results = $self->_dbh->selectall_arrayref( 'SELECT * FROM books',
+        { Columns => {} } );
+
+    my %site = (
+        host     => $self->{host},
+        def_lang => $self->{def_lang},
+    );
+
+    my $xml = XMLout( { site => \%site, record => $results }, NoAttr => 1 );
+
+    my $OUT_FILE;
+    open( $OUT_FILE, '>', $self->{dump_file} )
+        || croak(
+        maketext( "Could not open dump file for output: [_1]", $@ ) );
+
+    print( $OUT_FILE "<?xml version='1.0' encoding='utf-8' ?>\n" );
+    print( $OUT_FILE $xml );
+    close($OUT_FILE);
+
+    my $zip_file = $self->{dump_file} . '.zip';
+    unlink $zip_file if ( -f $zip_file );
+
+    if ( $self->{zip_dump} ) {
+        my $zip    = Archive::Zip->new();
+        my $member = $zip->addFile( $self->{dump_file} );
+        $member->desiredCompressionLevel(9);
+        $zip->writeToFileNamed($zip_file) == AZ_OK
+            || croak( maketext("dump file zip creation failed.") );
+    }
+
+    return;
+}
+
 1;    # Magic true value required at end of module
 __END__
 
@@ -1090,6 +1311,10 @@ Update an existing entry in the current Publican::WebSite DB...
 
 Returns number of rows added. Should be 1 for success, 0 for failure.
 
+=head2 update_or_add_entry
+
+Update an existing entry if it esists, else create a new entry.
+
 =head2  del_entry
 
 Delete an entry from the current Publican::WebSite DB...
@@ -1122,18 +1347,6 @@ Returns a string containing the current database statistics.
 
 Generate Site_Statistics.html containing the current database statistics.
 
-=head2 get_settings
-
-Get a reference to a hash for the settings for this site.
-
-=head2 update_settings
-
-Update the settings table in DB with supplied values.
-
-Values:
-	host   The host name
-    search HTML to use for navigation search.
-
 =head2 validate
 
 Validate the database entries against the RPM database.
@@ -1148,6 +1361,10 @@ Sort version strings in to correct order, handles X, X.Y, and X.Y.Z formats.
 
 Return the full toc path.
 
+=head2 xml_dump
+
+Generate an XML dump, and a zip of the dump if required, of the Database.
+
 =head1 DIAGNOSTICS
 
 =for author to fill in:
