@@ -2,6 +2,7 @@ package Publican::XmlClean;
 
 use strict;
 use warnings;
+use 5.008;
 use Carp;
 use XML::TreeBuilder;
 use Config::Simple;
@@ -639,10 +640,13 @@ sub print_xml {
 ##        $text =~ s/&amp;#x200B;/&#x200B;/g;
 ##        $text =~ s/&#x200B; &#x200B;/ /g;
 ##        $text =~ s/&#x200B; / /g;
-##        $text =~ s/&#60;/&lt;/g;
-##        $text =~ s/&#62;/&gt;/g;
-##        $text =~ s/&#34;/"/g;
-##        $text =~ s/&#39;/'/g;
+        $text =~ s/&#60;/&lt;/g;
+        $text =~ s/&#62;/&gt;/g;
+        $text =~ s/&#34;/"/g;
+        $text =~ s/&#39;/'/g;
+        $text =~ s/&quot;/"/g;
+        $text =~ s/&apos;/'/g;
+
         $xml_doc->root()->delete();
 
         my $OUTDOC;
@@ -836,22 +840,35 @@ sub my_as_XML {
                             if ($clean_id);
                         if ( -f $img_file ) {
                             my ( $width, $height ) = imgsize($img_file);
-                            if ( $@ || !$width ) {
-                                logger(
-                                    maketext(
-                                        "Can't calculate image size, image may render badly. Image File: [_1]. Error Message: [_2]",
-                                        $img_file, $@
-                                        )
-                                        . "\n"
-                                );
+                            my $max_width
+                                = $self->{publican}->param('max_image_width')
+                                || 10000;
+                            my $set_width = $node->attr('width') || 0;
+                            $set_width =~ s/[^\d]//g;
+
+                            if ( $set_width == 444 ) {
+                                $node->attr( 'width', undef );
+                                $set_width = 0;
                             }
-                            elsif ( $width
-                                > $self->{publican}->param('max_image_width')
-                                )
-                            {
-                                $node->attr( 'width',
-                                    $self->{publican}
-                                        ->param('max_image_width') );
+
+                            # Do not override user settings
+                            if ($set_width == 0) {
+                                $max_width = $set_width
+                                    if ( $set_width
+                                    && $set_width < $max_width );
+
+                                if ( $@ || !$width ) {
+                                    logger(
+                                        maketext(
+                                            "Can't calculate image size, image may render badly. Image File: [_1]. Error Message: [_2]",
+                                            $img_file, $@
+                                            )
+                                            . "\n"
+                                    );
+                                }
+                                elsif ( $width > $max_width ) {
+                                    $node->attr( 'width', $max_width );
+                                }
                             }
                         }
                         elsif ( $img_file !~ /Common_Content/ ) {
